/*
 * Copyright 2017 Cray Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 @description Global data used by unit tests.
 */

/* exported BASE_NODE_CONTROLLER_PROPERTIES */

var BASE_NODE_CONTROLLER_PROPERTIES = [
  '_getImages',
  '_loadDrivers',
  '_sortDriverProperties',
  'cancel',
  'collectionCheckPropertyUnique',
  'collectionDeleteProperty',
  'driverInterfaceFields',
  'driverProperties',
  'driverPropertyGroups',
  'drivers',
  'images',
  'isDriverPropertyActive',
  'loadDriver',
  '_loadDriverDetails',
  '_loadDriverProperties',
  'loadingDriverProperties',
  'modalTitle',
  'node',
  'propertyCollections',
  'readyToSubmit',
  'submitButtonTitle',
  'validHostNameRegex',
  'driverType'];

/* exported BASE_PORT_CONTROLLER_PROPERTIES */

var BASE_PORT_CONTROLLER_PROPERTIES = [
  'address',
  'cancel',
  'extra',
  'localLinkConnection',
  'pxeEnabled',
  'portgroup_uuid'];

/* exported BASE_PORTGROUP_CONTROLLER_PROPERTIES */

var BASE_PORTGROUP_CONTROLLER_PROPERTIES = [
  'address',
  'cancel',
  'extra',
  'mode',
  'name',
  'properties',
  'standalone_ports_supported'];

/* exported PROPERTY_COLLECTION_PROPERTIES */

var PROPERTY_COLLECTION_PROPERTIES = [
  'id',
  'formId',
  'title',
  'addPrompt',
  'placeholder'
];
